use super::{PossessionData, PossessionType};
use crate::{
    message_handler::user_commands::{UResult, UserError},
    models::item::{Item, ItemFlag},
    static_content::possession_type::{ContainerCheck, ContainerData},
};
use once_cell::sync::OnceCell;
use std::collections::BTreeSet;

pub fn recipe_set() -> &'static BTreeSet<PossessionType> {
    static SET: OnceCell<BTreeSet<PossessionType>> = OnceCell::new();
    &SET.get_or_init(|| {
        vec![PossessionType::GrilledSteakRecipe]
            .into_iter()
            .collect()
    })
}

struct RecipesOnlyChecker;
impl ContainerCheck for RecipesOnlyChecker {
    fn check_place(&self, _container: &Item, item: &Item) -> UResult<()> {
        if !self.check_place_type(&item.possession_type) {
            Err(UserError(
                "You don't find a sensible place for that in a recipe book.".to_owned(),
            ))
        } else {
            Ok(())
        }
    }
    fn check_place_type(&self, item_type: &Option<PossessionType>) -> bool {
        item_type
            .as_ref()
            .map_or(false, |pt| recipe_set().contains(pt))
    }
}

static RECIPES_ONLY_CHECKER: RecipesOnlyChecker = RecipesOnlyChecker;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (PossessionType::CulinaryEssentials,
         PossessionData {
             display: "Culinary Essentials cookbook",
             aliases: vec!["book", "cookbook"],
             details: "A weathered cookbook filled with essential recipes for survival in the post-apocalyptic world. Its pages are yellowed and marked with stains, a testament to its extensive use by those seeking sustenance and comfort amidst scarcity.",
             weight: 300,
             container_data: Some(ContainerData {
                 base_weight: 290,
                 default_contents: vec![
                     PossessionType::GrilledSteakRecipe
                 ],
                 checker: &RECIPES_ONLY_CHECKER,
                 ..Default::default()
             }),
             default_flags: vec![ItemFlag::Book],
             ..Default::default()
         }),
        (PossessionType::GrilledSteakRecipe,
         PossessionData {
             display: "recipe for Grilled Steak",
             aliases: vec!["grilled steak", "recipe"],
             details: "Instructions for how to make a basic but mouthwatering steak",
             weight: 10,
             default_flags: vec![ItemFlag::Instructions],
             ..Default::default()
         }),
        (PossessionType::EverydayChemistry,
         PossessionData {
             display: "Everyday Chemistry",
             aliases: vec!["book", "chemistry book"],
             details: "A weathered collection of instructions on various forms of chemical synthesis",
             weight: 310,
             container_data: Some(ContainerData {
                 base_weight: 290,
                 default_contents: vec![
                     PossessionType::VenomExtractionKitInstructions,
                     PossessionType::SuperchargeTraumaKit,
                 ],
                 checker: &RECIPES_ONLY_CHECKER,
                 ..Default::default()
             }),
             default_flags: vec![ItemFlag::Book],
             ..Default::default()
         }),
        (PossessionType::VenomExtractionKitInstructions,
         PossessionData {
             display: "venom extraction kit instructions",
             aliases: vec!["recipe", "instructions"],
             details: "Instructions for how to manufacture a special polymer that sucks all the venom from a bite or sting site",
             weight: 10,
             default_flags: vec![ItemFlag::Instructions],
             ..Default::default()
         }),
        (PossessionType::SuperchargeTraumaKit,
         PossessionData {
             display: "how to supercharge a trauma kit",
             aliases: vec!["trauma kit", "large trauma kit", "supercharge a trauma kit", "recipe", "instructions"],
             details: "A detailed written procedure for supercharging a trauma kit. Apparently, the empire deliberately reduced the capacity of the batteries in the equipment in the kit to keep people under its thumb, but a simple replacement of some alkaline solution in the batteries can double the number of treatments",
             weight: 10,
             default_flags: vec![ItemFlag::Instructions],
             ..Default::default()
         }),
    ))
}
